unit uPlugin;

interface
uses Windows, DVstUtils, uEditor, DAEffect, DAEffectX,
     DAudioEffect, DAudioEffectX, DVstTemplate, Forms,
     Math, DDspUtils, Sysutils, Dialogs;

type
 APlugin = class(TVSTTemplate)
  private
 public
  procedure processMIDI(ev: VstMidiEvent); override;
  procedure initializeParameters; override;
  constructor Create(audioMaster: TAudioMasterCallbackFunc);
  procedure editorIdle(Sender: TObject);
  procedure processAudio(const inputs, outputs: TArrayOfSingleArray;
   sampleframes: Integer); override;
  procedure parameterChanged(Index: Integer);

 end;

implementation

//******************************************************************************
// V konstruktoru Create jsou definovny zkladn parametry budoucho modulu.
//******************************************************************************

constructor APlugin.Create(audioMaster: TAudioMasterCallbackFunc);
var numPrograms, numParameters: integer;
begin
 canDos := [receiveVstEvents, receiveVstMidiEvent,
  sendVstEvents, sendVstMidiEvent, receiveVstTimeInfo, sendVstTimeInfo];
 Properties := [prCanMono, prCanReplacing];
 numInputs := 2;                 // Poet audio vstup
 numOutputs := 2;                // Poet audio vstup
 numPrograms := 1;               // Poet program
 numParameters := 9;             // Poet parametr
 UniqueID := 'TUNL';             // Identifiktor modulu
 EffectName := 'VSTunel';        // Nzev modulu
 ProductName := 'VSTunel';
 VendorName := 'Jan Valendin';
 VendorVersion := 1;
 initialDelay := 0;
 tailSize := 0;

//******************************************************************************
// TPluginEditorWindow odkazuje na formul editoru, nahradme-li jej ukazatelem
// nil, bude modul pracovat bez vlastnho grafickho uivatelskho rozhran
//******************************************************************************

 inherited Create(audiomaster, numPrograms, numParameters, TPluginEditorWindow);
 OnEditorIdle := EditorIdle;
 OnParameterChange := ParameterChanged;
end;

//******************************************************************************
// V procedue APlugin.initializeParameters je mono definovat vlastnosti
// parametr. Pokud to uivatel neudl, rozsah hodnot se nastav na 0..1
//******************************************************************************

procedure APlugin.initializeParameters;
begin

 ParameterProperties[0].min := -24;
 ParameterProperties[0].max := 24;
 ParameterProperties[0].name := 'Transpose';
 ParameterProperties[0].units := 'semitones';
 ParameterProperties[0].curve := ctLinear;

 setParameterEx(0, 0);

 ParameterProperties[1].min := -127;
 ParameterProperties[1].max :=  127;
 ParameterProperties[1].name := 'Velocity shift';
 ParameterProperties[1].curve := ctLinear;
 setParameterEx(1, 0);

 ParameterProperties[2].min := 0;
 ParameterProperties[2].max := 127;
 ParameterProperties[2].name := 'Parameter1';
 ParameterProperties[2].curve := ctLinear;
 setParameterEx(2, 100);

 ParameterProperties[3].min := 0;
 ParameterProperties[3].max := 127;
 ParameterProperties[3].name := 'Parameter2';
 ParameterProperties[3].curve := ctLinear;
 setParameterEx(3, 100);

 ParameterProperties[4].min := 0;
 ParameterProperties[4].max := 127;
 ParameterProperties[4].name := 'Parameter3';
 ParameterProperties[4].curve := ctLinear;
 setParameterEx(4, 100);

 ParameterProperties[5].min := 0;
 ParameterProperties[5].max := 127;
 ParameterProperties[5].name := 'Parameter4';
 ParameterProperties[5].curve := ctLinear;
 setParameterEx(5, 100);

 ParameterProperties[6].min := 0;
 ParameterProperties[6].max := 127;
 ParameterProperties[6].name := 'Switch1';
 setParameterEx(6, 127);

 ParameterProperties[7].min := 0;
 ParameterProperties[7].max := 127;
 ParameterProperties[7].name := 'Switch2';
 setParameterEx(7, 0);

 ParameterProperties[8].min := 0;
 ParameterProperties[8].max := 127;
 ParameterProperties[8].name := 'ForNoteOff';
 setParameterEx(8, 0);



end;

//******************************************************************************
// Procedura APlugin.processMIDI slou ke zpracovn pchozch zprv MIDI
//******************************************************************************

procedure APlugin.processMIDI(ev: VstMidiEvent);
var newnote, newvelo, time, data1, data2, status, channel: integer;
begin
 channel := ev.midiData[0] and $0F;
 status := ev.midiData[0] and $F0;
 data1 := ev.midiData[1] and $7F;
 data2 := ev.midiData[2] and $7F;
 time := ev.deltaFrames;

 if (status = $90) and (data2 > 0) then // Zprva MIDI typu Stisknut klvesy
 begin
  // data1 obsahuje slo noty
  // data2 obsahuje rychlost stlaen klvesy
  newnote := round(f_limit(data1 + getParameterEx(0), 0, 127));
  newvelo := round(f_limit(data2 + getParameterEx(1), 0, 127));
  MIDI_NoteOn(channel, newnote, newvelo , time);
 end
 else if ((status = $90) and (data2 = 0)) or
  (status = $80) then // Note Off
 begin
  // data1 obsahuje slo noty
  // data2 obsahuje rychlost uvolnn klvesy
  // pole zprvu typu "Note Off" zpt hostitelskmu programu MIDI thru)

  newnote := round(f_limit(data1 + getParameterEx(0), 0, 127));
  newvelo := round(f_limit(data2 + getParameterEx(1), 0, 127));
  MIDI_NoteOff(channel, newnote, newvelo, time);  //
 end
 else if (status = $A0) then // Zprva MIDI typu Polyphonic Aftertouch
 begin
  MIDI_PolyAftertouch(channel, data1, data2);
  // data1 obsahuje slo noty
  // data2 obsahuje hodnotu pro aftertouch
 end
 else if (status = $B0) then // Zprva MIDI typu MIDI Controller
 begin
  MIDI_CC(channel,data1, data2);

  // data1 obsahuje slo ovladae CC
  // data2 obsahuje hodnotu
 end
 else if (status = $C0) then // Zprva MIDI typu Program Change
 begin
  MIDI_ProgramChange(status,data1);
  // data1 obsahuje slo programu
 end
 else if (status = $D0) then // Zprva MIDI typu Channel Aftertouch
 begin
  MIDI_ChannelAftertouch(channel,data1);
  // data1 obsahuje hodnotu pro aftertouch
 end
 else if (status = $E0) then // Zprva MIDI typu Pitchbend
 begin
  MIDI_PitchBend2(channel,data1,data2);
  // data1 a data2 dohromady tvo 12b hodnotu pro ohbn tnu
 end;
end;

//******************************************************************************
// Skrze proceduru editorIdle je mono pistupovat z vlkna modulu k ovldacm
// prvkm uivatelskho rozhran (tj. k vlknu editoru).
//******************************************************************************

procedure APlugin.editorIdle(Sender: TObject);
begin
 with (Sender as TPluginEditorWindow) do
 begin
  // Pepis aktulnch hodnot parametr do grafickch popisk

  label16.Caption:=inttostr(round(getParameterEx(0)));
  label17.Caption:=inttostr(round(getParameterEx(1)));
  label18.Caption:=inttostr(round(getParameterEx(2)));
  label19.Caption:=inttostr(round(getParameterEx(3)));
  label20.Caption:=inttostr(round(getParameterEx(4)));
  label21.Caption:=inttostr(round(getParameterEx(5)));
{  UpDown5.Position:=round(getParameterEx(2));
  UpDown6.Position:=round(getParameterEx(3));
  UpDown7.Position:=round(getParameterEx(4));
  UpDown8.Position:=round(getParameterEx(5));
}
 end;
end;

//******************************************************************************
// Procedura APlugin.procesAudio provd zpracovn zvukovch vstup, v tomto
// ppad pouze ped vstupn data v nezmnn podob na vstup. Je zde
// implemetovna pouze z dvodu kompatibility s hostitelskmi programy.
//******************************************************************************

procedure APlugin.processAudio(const inputs, outputs: TArrayOfSingleArray;
  sampleframes: Integer);
var i: integer;
begin
 for i := 0 to sampleframes - 1 do
 begin
  outputs[0, i] := inputs[0, i];
  outputs[1, i] := inputs[1, i];
 end;
end;

//******************************************************************************
// Procedura parameterChanged je reakc na zmnu nkterho z parametr.
// Pokud se jedn o parametr ovldajc nkter z MIDI Controller Change kanl,
// dojde k vygenerovn odpovdajc zprvy MIDI.
//******************************************************************************

procedure APlugin.parameterChanged(Index: Integer);
//var i:integer;
begin
 if (Index = 2) then
    begin
    MIDI_CC(1,55,round(getParameterEx(2)));
    end else
 if (Index = 3) then
    begin
    MIDI_CC(1,56,round(getParameterEx(3)));
    end else
 if (Index = 4) then
    begin
    MIDI_CC(1,57,round(getParameterEx(4)));
    end else
 if (Index = 5) then
    begin
    MIDI_CC(1,58,round(getParameterEx(5)));
    end else
 if (Index = 6) then
    begin
    MIDI_CC(1,59,round(getParameterEx(6)));
    end else
 if (Index = 7) then
    begin
    MIDI_CC(1,60,round(getParameterEx(7)));
    end else
 if (Index = 8) then
    begin
    if (getParameterEx(8)<127) then
       begin
       MIDI_NoteOff(1,round(getParameterEx(8)),60);
       setParameterEx(8,getParameterEx(8)+1)
       end;
    end;
end;

end.


